#######################################################################################
# Author: b.arendarczyk@sms.ed.ac.uk
# Purpose: Sample posterior distribution of parameters in PLUMv2 using
# Approximate Bayesian Sampling with Markov Chain Monte Carlo (ABC-MCMC)
#######################################################################################

# LAND_CHANGE_COST = function() {runif(1, 0, 1)}
# CROP_INCREASE_FACTOR = function() {runif(1, 0, 2)}
# CROP_DECREASE_FACTOR = function() {runif(1, 0, 2)}
EPSILON <- 0.5 # proposal acceptance threshold
SIGMA <- 0.1 # SD of proposal kernel
BASE_DIR <- "/exports/csce/eddie/geos/groups/LURG/models/PLUM/output/abc_mcmc"

land_covers <- c("cropland", "pasture", "timberForest", "carbonForest", "natural")

start_values <- data.frame(fromLC = character(), toLC = character(), Cost = numeric(),
                           stringsAsFactors = F)

# Generate land cover change pairs
n_lc <- length(land_covers)
for (i in 1:n_lc) {
  for (j in 1:n_lc) {
    start_values[(i - 1) * n_lc + j, 1:2] <- c(land_covers[i], land_covers[j])
    start_values[(i - 1) * n_lc + j, 3] <- 0.5
  }
}

# Write land cover change file
write_param_file <- function(proposal, param_dir) {
  prop_vals <- start_values
  prop_vals$Cost <- proposal
  param_file <- file.path(param_dir, "conversion_costs.csv")
  write.csv(prop_vals, param_file, row.names = F)
}

# Calculate output statistics
calc_stats <- function(out_dir) {
  
}

# Calculate distance of stats between generated and observed
calc_dist <- function(stats) {
  
}

run_ABC_MCMC <- function(start_values, iterations) {
    chain <- array(dim = c(iterations + 1, nrow(start_values)))
    chain[1, ] <- start_values[, 3]
    
    for (i in 1:iterations) {
      # Generate new proposal for params
      proposal <- rnorm(nrow(start_values), mean = chain[i,], sd = SIGMA)
      
      # Make new output dir
      out_folder <- paste0("iter", i)
      out_dir <- file.path(BASE_DIR, out_folder)
      if (dir.exists(out_dir)) {
        stop("Output directory already exists. Terminating.")
      }
      dir.create(out_dir)
      
      calib_dir <- file.path(BASE_DIR, out_folder, "calib")
      scen_dir <- file.path(BASE_DIR, out_folder, "scen")
      dir.create(calib_dir)
      dir.create(scen_dir)
      
      # Write proposed params to file
      write_param_file(proposal, calib_dir)
      write_param_file(proposal, scen_dir)
      
      # Set up config files
      base_config_path <- file.path(BASE_DIR, "config.properties")
      
      # Calib config
      new_config_path <- file.path(calib_dir, "config.properties")
      file.copy(base_config_path, new_config_path)
      write(paste0("CONVERSION_COST_FILE=", file.path(calib_dir, "conversion_costs.csv")), new_config_path, append=T) 
      write(paste0("OUTPUT_DIR=", calib_dir), new_config_path, append=T) 
      write("IS_CALIBRATION_RUN=true", new_config_path, append=T) 
      
      # Copy cluster file
      cluster_file <- file.path(BASE_DIR, "cluster.asc")
      file.copy(cluster_file, calib_dir)
      
      # Scenario config
      new_config_path <- file.path(scen_dir, "config.properties")
      file.copy(base_config_path, new_config_path)
      write(paste0("CONVERSION_COST_FILE=", file.path(scen_dir, "conversion_costs.csv")), new_config_path, append=T) 
      write(paste0("OUTPUT_DIR=", scen_dir), new_config_path, append=T) 
      write("IS_CALIBRATION_RUN=false", new_config_path, append=T) 
      write(paste0("CALIB_DIR=", calib_dir), new_config_path, append=T) 

      # Run calibration
      calib_folder <- file.path("abc_mcmc", out_folder, "calib")
      calib_job_name <- paste0("calib", i)
      run_plum_str <- paste("qsub -N", calib_job_name, "/exports/csce/eddie/geos/groups/LURG/models/PLUM/plumv2/scripts/runPlum.sh -s", calib_folder, "-r y -p n -b Bart")
      system(run_plum_str)
      
      # Run scenario
      scen_folder <- file.path("abc_mcmc", out_folder, "scen")
      scen_job_name <- paste0("scen", i)
      run_plum_str <- paste("qsub -N", scen_job_name, "-hold_jid", calib_job_name,"/exports/csce/eddie/geos/groups/LURG/models/PLUM/plumv2/scripts/runPlum.sh -s", scen_folder, "-r y -p n -b Bart")
      system(run_plum_str)
      
      return()
      # Generate summary statistics {s1, s2, ...}
      stat <- calc_stats(out_dir)
      
      # Calculate distance between observed and simulated statistics d
      d <- calc_dist(stat)
      
      # Accept or reject proposal
      if (d < EPSILON) {
        chain[i+1, ] <- proposal
      } else {
        chain[i+1, ] <- chain[i, ]
      }
    }
    return(chain)
}


run_ABC_MCMC(start_values, 1)
